#!/usr/bin/env ruby
# screencap_to_data.rb - Convert a screen capture of the game Mario's
# Picross into a datafile suitable for Peter's Picross.
#
# Copyright (C) 2008 Conrad Meyer
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Usage:
# 1. put all the screenshots (x-y_Foo.png) in ./input/ (where this script is in .)
# 2. mkdir output
# 3. ruby ./screencap_to_data.rb
# 4. Look at .txt files in output/

require 'rubygems'
require 'RMagick'
include Magick

# 1) crop to 178x178+116+105
def step1(img)
  img.crop(116, 105, 178, 178)
end

# 2) check every 6 pixels for 0 intensity (black)
def step2(img)
  ary = img.export_pixels(0, 0, img.columns, img.rows, "I")
  res = []
  0.upto(14) do |row|
    res[row] = []
    0.upto(14) do |col|
      res[row][col] = (ary[row*12*178 + col*12] == 0)? 1 : 0
    end
  end
  res
end

# 3) Reduce 15x15 grid to minimum square that will fit all the black pixels
def step3(squares)
  size = 15
  15.downto(5) do |dim|
    row = squares[dim-1].dup
    col = squares.map do |row| row[dim-1] end
    if row.include?(1) || col.include?(1)
      size = dim
      break
    end
  end

  # remove the last row(s)
  squares.delete_at(squares.length-1) while squares.length > size
  # col(s)
  squares.each do |row|
    row.delete_at(row.length-1) while row.length > size
  end
  squares
end

# 4) Write to file
def step4(fn, smallened)
  newfn = fn.sub /^input/, "output"
  newfn = newfn + ".txt"
  fn =~ %r|^input/(\d+)\-(\d+)_([^\.]+)\.png$|
  level1, level2, name = $1, $2, $3

  open(newfn, "w") do |file|
    file.puts "ppp"
    file.puts "Nintendo"
    file.puts name[0...20]    # first 20 characters of name, max
    file.puts smallened.length.to_s
    file.puts "kin_#{level1}_#{level2}"

    smallened.each do |row|
      file.puts row.to_s
    end
  end
end

# main:

Dir["input/*.png"].each do |png|
  img = step1(ImageList.new(png)[0])
  squares = step2(img)
  smallened = step3(squares)
  step4(png, smallened)
end
