 /*
    Peter's Picross - A remake for TI 68k-based calculators of the Nintendo Gameboy game, "Mario's Picross"
    Copyright (C) 2008  Peter Fernandes
    supersonicandtails@gmail.com

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
    
    
    main.c: skeleton of game branching from the main menu
*/


#include <tigcclib.h>
#include "main.h"
#include "engine.h"
#include "files.h"
#include "editor.h"
#include "menus.h"



//REMINDER: Tell people about game secret


// Main Function
void _main(void) {
	short delaysave0=OSInitKeyInitDelay(50);
	short delaysave1=OSInitBetweenKeyDelay(20);
	unsigned char font;
	short err;
	short err1=0;
	short menu_result;
	char *pos;
	LCD_BUFFER screen;
	
	
	LCD_save(screen);
	
	
	randomize();
	
	font=FontGetSys();
	
	settings_file_isarchived=0;
	
	
	if((err=create_file("picset",&settings_file,103,&settings_handle))) {
		if(err==-1) {
			//open the existing file
			if(!(settings_file=open_file("picross\\picset",&settings_handle))) {
				//couldn't open the settings file
				err1=1;
				goto skip;
			}
		}
		else {
			//throw an error
			err1=2;
			goto skip;
		}
		
	}
	else {
		pos=settings_file;
		
		//file size
		*(((unsigned short *)pos)++)=21+72+1+7;
		
		//cue past the size entry just like open_file
		settings_file+=2;
		
		//initialize the author string
		*pos=0;
		
		pos+=21;
		
		//now the save state section for the teacher key
		*pos=0;//filename of puzzle (9), seconds (2), board cleared (15), board marked (15), padding (1)
		pos+=72;
		
		
		
		
		//number of puzzle records in the file
		*(pos++)=0;
		

		*(pos++)=0;
		*(pos++)='P';
		*(pos++)='I';
		*(pos++)='C';
		*(pos++)='S';
		*(pos++)=0;
		*(pos)=OTH_TAG;
	
		
	}

		
	PUZZLE puzzle;
	
	top:
			
		
	switch((menu_result=main_menu())) {
		case 49:
		case 39:
			if((err=create_puzzle(menu_result==39?1:0))) {
				clrscr();
				
				if(err<-1)
					printf("Could not create puzzle:\n");
				
				switch (err) {
					case -1:
						goto top;
					case -2:
						printf("Not enough memory");
						break;
					case -3:
						printf("Error creating VAT symbol");
						break;
					case -4:
						printf("Error dereferencing VAT symbol");
						break;
					case -5:
						printf("Error dereferencing file data");
					case -6:
						goto skip;
				}
			}
			else {
				clrscr();
				printf("Puzzle Created!");
				ngetchx();
				goto top;
			}
			
			break;
			
		case 29:
		case 19:
		case 59:
			if(menu_result==59 && *(settings_file+21)) {
				err=dialog_load_puzzle(&puzzle,0,settings_file+21);
			}
			else {
				if(menu_result==59)
					goto top;
				err=dialog_load_puzzle(&puzzle,menu_result==19?-1:0,NULL);
			}
			
			clrscr();
			
			if(err<-1)
				printf("Could not load puzzle:\n");
			
			switch(err) {
				case -2:
					printf("No puzzles found in folder \"picross\"");
					break;
				case -3:
					printf("Could open file");
					break;
				case -4:
					printf("Error: Memory");
					break;
				case -5:
					goto top;
				case -6:
					printf("Puzzle is blank");
					break;
				default:
					if(main_loop(&puzzle,menu_result==59?MODE_RESTORE:MODE_PLAY,NULL))
						goto skip; //teacher key
					goto top;
			}
			
			break;
		
		case 69:
			credits();
			goto top;
		
		default:
			goto skip;
	}
	
	ngetchx();
	
	goto top;
	
	skip:
	
	err=0;
	
		
	FontSetSys(font);
	OSInitKeyInitDelay(delaysave0);
	OSInitBetweenKeyDelay(delaysave1);
	
	HeapUnlock(settings_handle);
	
	LCD_restore(screen);
	
	switch (err1) {
		case 1:
			ST_helpMsg("Error: Couldn't open settings file");
			break;
		case 2:
			ST_helpMsg("Error: Couldn't create settings file");
	}

}
